#!/bin/sh
#find-debuginfo.sh - automagically generate debug info and file list
#for inclusion in an rpm spec file.

if [ -z "$1" ] ; then BUILDDIR="."
else BUILDDIR=$1
fi

LISTFILE=$BUILDDIR/debugfiles.list
SOURCEFILE=$BUILDDIR/debugsources.list

echo -n > $SOURCEFILE

binaries=`find $RPM_BUILD_ROOT -type f \( -perm +111 -or -name "*.ko" \)`
# Strip ELF binaries
for f in $binaries
do
	file_output=`file "$f"`
	case $file_output in
	    *ELF*", not stripped")
		;;
	    *ELF*", stripped")
		echo "WARNING: "`echo $f | sed -e "s,^$RPM_BUILD_ROOT/*,/,"`" is already stripped!"
		continue
		;;
	    *)
		continue
		;;
	esac
	BASEDIR=`dirname $f | sed -n -e "s#^$RPM_BUILD_ROOT#/#p"`
	[ "$BASEDIR" = "/usr/lib/debug" ] && continue;
	OUTPUTDIR=${RPM_BUILD_ROOT}/usr/lib/debug${BASEDIR}
	mkdir -p ${OUTPUTDIR}
	echo extracting debug info from $f
	mode=$(stat -c %a $f)
	chmod +w $f
	/usr/lib/rpm/debugedit -b $RPM_BUILD_DIR -d /usr/src/debug -l $SOURCEFILE $f
        outfile=$OUTPUTDIR/`basename $f`.debug
        objcopy --only-keep-debug $f $outfile || :
	strip_option="--strip-all"
	case "$f" in
	    *.ko)
		strip_option="--strip-debug"
		;;
	esac
	if test -n "$STRIP_KEEP_SYMTAB"; then
	    strip_option="--strip-debug"
	fi
	objcopy --add-gnu-debuglink=$outfile ${strip_option} $f || :
	chmod $mode $f
done

mkdir -p ${RPM_BUILD_ROOT}/usr/src/debug
(cd $RPM_BUILD_DIR; LANG=C sort -z -u | cpio -pd0m ${RPM_BUILD_ROOT}/usr/src/debug) < $SOURCEFILE
# stupid cpio creates new directories in mode 0700, fixup
# don't package world writeable files
chmod -R a+rX,go-w ${RPM_BUILD_ROOT}/usr/src/debug

# trying to replace dangling and/or absolute symlinks
DBASE=${RPM_BUILD_ROOT}/usr/src/debug 
links=`find $DBASE -type l -printf "%P\n"`
for link in $links; do
	link_file=`readlink -f $RPM_BUILD_DIR/$link`
	if ! test -e $DBASE/$link || # dangling
	   case $link_file in /*) true;; *) false;; esac; then
		rm -f $DBASE/$link
		cp $link_file $DBASE/$link
	fi
done

{
	test -d ${RPM_BUILD_ROOT}/usr/lib/debug && echo /usr/lib/debug
	echo /usr/src/debug
} > $LISTFILE
